from django import forms

from discussion.models import Comment, Post, Discussion


class CommentForm(forms.ModelForm):
    class Meta:
        exclude = ('user', 'post')
        model = Comment
        widgets = {
            'body' : forms.Textarea(attrs={'placeholder' : 'Reply to this conversation'}),
        }


class PostForm(forms.ModelForm):
    class Meta:
        exclude = ('user', 'discussion')
        model = Post
        widgets = {
            'body' : forms.Textarea(attrs={'placeholder' : 'Start a conversation'}),
        }

class SearchForm(forms.Form):
    search = forms.CharField()
    discussion = forms.ModelChoiceField(required=False, queryset=Discussion.objects, empty_label='All discussions')
    #class Meta:
    #    exclude = ('discussion', 'slug')
    #    model = Post
    


import re
from django import template
from discussion.forms import CommentForm
from django.utils.safestring import mark_safe

re_img = re.compile(r'\.(bmp|jpe?g|jp2|jxr|gif|png|tiff?)$', re.IGNORECASE)

register = template.Library()

@register.inclusion_tag('discussion/limit_comments.html', takes_context=True)
def limit_comments(context, post, first=2, last=1, limit=None, comments=None):
    """
    Render a limited set of comments for a post.
        first: The number of comments to display from the start of the list.
        last: The number of comments to display from the end of the list.
        limit: If there are less comments than this limit then display all comments.
            This defaults to first + last.
        comments: A list of comments to use (The default is the posts comments).

    {% limit_comments post 3 2 %}
    """
    
    if limit is None:
        limit = first+last
    if comments is None:
        comments = post.comment_set.all().select_related('post', 'post__discussion')

    count = comments.count()

    if count > limit:
        first_comments = comments[:first]
        last_comments = comments[count-last:]
        hidden_count = count-first-last
    else:
        first_comments = comments
        last_comments = None
        hidden_count = 0

    context.update({
        'post': post,
        'all_comments': comments,
        'first_comments': first_comments,
        'last_comments': last_comments,
        'all_count': count,
        'hidden_count': hidden_count,
    })

    return context


@register.inclusion_tag('discussion/comment_form.html')
def comment_form(post):
    """
    Render the comment form with the correct prefix.
    """
    return {
        'post': post,
        'form': CommentForm(prefix=post.prefix),
    }


@register.filter
def is_image(value):
    """
    Is the file an image (based on it's extension)
    """
    return re_img.search(value.name)


@register.filter
def highlight(text, word):
    return mark_safe(re.compile('(?P<word>%s)' % re.escape(word), re.IGNORECASE).sub("<span class='highlight'>\g<word></span>", text))




# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Discussion'
        db.create_table('discussion_discussion', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=50)),
        ))
        db.send_create_signal('discussion', ['Discussion'])

        # Adding model 'Post'
        db.create_table('discussion_post', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('discussion', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['discussion.Discussion'])),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('body', self.gf('django.db.models.fields.TextField')()),
            ('attachment', self.gf('django.db.models.fields.files.FileField')(max_length=100, null=True, blank=True)),
            ('time', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('discussion', ['Post'])

        # Adding model 'Comment'
        db.create_table('discussion_comment', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('post', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['discussion.Post'])),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('body', self.gf('django.db.models.fields.TextField')()),
            ('attachment', self.gf('django.db.models.fields.files.FileField')(max_length=100, null=True, blank=True)),
            ('time', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('discussion', ['Comment'])

    def backwards(self, orm):
        # Deleting model 'Discussion'
        db.delete_table('discussion_discussion')

        # Deleting model 'Post'
        db.delete_table('discussion_post')

        # Deleting model 'Comment'
        db.delete_table('discussion_comment')

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'discussion.comment': {
            'Meta': {'ordering': "('time',)", 'object_name': 'Comment'},
            'attachment': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'body': ('django.db.models.fields.TextField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'post': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['discussion.Post']"}),
            'time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'discussion.discussion': {
            'Meta': {'object_name': 'Discussion'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'discussion.post': {
            'Meta': {'object_name': 'Post'},
            'attachment': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'body': ('django.db.models.fields.TextField', [], {}),
            'discussion': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['discussion.Discussion']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        }
    }

    complete_apps = ['discussion']


from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.views.generic import CreateView, DetailView, ListView, FormView
from django.views.generic.list import BaseListView
from django.db.models import Q

from discussion.forms import CommentForm, PostForm, SearchForm
from discussion.models import Discussion, Comment, Post
from discussion.utils import class_view_decorator


class SearchFormMixin(object):
    """Add the basic search form to your view."""

    search_initial = {}

    def get_context_data(self, **kwargs):
        context = super(SearchFormMixin, self).get_context_data(**kwargs)
        context['search_form'] = self.get_search_form(SearchForm)
        return context

    def get_search_form(self, form_class):
        """
        Returns an instance of the search form to be used in this view.
        """
        return form_class(**self.get_search_form_kwargs())

    def get_search_form_kwargs(self):
        """
        Returns the keyword arguments for instanciating the search form.
        """
        kwargs = {'initial': self.get_search_initial()}
        if self.request.method in ('POST', 'PUT'):
            kwargs.update({
                'data': self.request.POST,
                'files': self.request.FILES,
            })

        return kwargs

    def get_search_initial(self):
        """
        Returns the initial data to use for search forms on this view.
        """
        return self.search_initial


class DiscussionMixin(object):
    discussion_slug = 'discussion_slug'

    def dispatch(self, request, *args, **kwargs):
        self.discussion = self.get_discussion(slug=kwargs[self.discussion_slug])
        return super(DiscussionMixin, self).dispatch(request, *args, **kwargs)

    def get_discussion(self, slug=None):
        if slug is None:
            slug = self.kwargs[self.discussion_slug]
        return Discussion.objects.get(slug=slug)

    def get_queryset(self):
        qs = super(DiscussionMixin, self).get_queryset()
        return qs.filter(discussion=self.discussion)


@class_view_decorator(login_required)
class DiscussionList(SearchFormMixin, ListView):
    model = Discussion


@class_view_decorator(login_required)
class DiscussionView(SearchFormMixin, DetailView):
    model = Discussion

    def get_context_data(self, **kwargs):
        self.search_initial.update({'discussion': self.object})
        context = super(DiscussionView, self).get_context_data(**kwargs)
        context['form'] = PostForm()
        return context


@class_view_decorator(login_required)
class CreatePost(DiscussionMixin, CreateView):
    form_class = PostForm
    model = Post

    def get_form_kwargs(self):
        kwargs = super(CreatePost, self).get_form_kwargs()
        instance = self.model(user=self.request.user, discussion=self.discussion)
        kwargs.update({'instance': instance})
        return kwargs

    def get_context_data(self, **kwargs):
        context = super(CreatePost, self).get_context_data(**kwargs)
        context['discussion'] = self.discussion
        return context

@class_view_decorator(login_required)
class PostView(DiscussionMixin, CreateView):
    form_class = CommentForm
    model = Comment
    template_name = 'discussion/post_detail.html'
    ajax_form_valid_template_name = 'discussion/_comment_detail.html'
    ajax_form_invalid_template_name = 'discussion/comment_form.html'

    def dispatch(self, request, *args, **kwargs):
        self.post_obj = self.get_post(kwargs['pk'])
        return super(PostView, self).dispatch(request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super(PostView, self).get_form_kwargs()
        instance = self.model(user=self.request.user, post=self.post_obj)
        kwargs.update({
            'instance': instance,
            'prefix': self.post_obj.prefix,
        })
        return kwargs

    def get_context_data(self, **kwargs):
        context = super(PostView, self).get_context_data(**kwargs)
        context['post'] = self.post_obj
        return context

    def get_post(self, pk=None):
        if pk is None:
            pk = self.kwargs['pk']
        return Post.objects.get(pk=pk)

    def get_success_url(self):
        kwargs = {
            'pk': self.kwargs['pk'],
            'discussion_slug': self.kwargs[self.discussion_slug]
        }
        return reverse('discussion_post', kwargs=kwargs)

    def form_valid(self, form):
        if self.request.is_ajax():
            self.object = form.save()
            self.template_name = self.ajax_form_valid_template_name
            return self.render_to_response(self.get_context_data(comment=self.object))
        else:
            return super(PostView, self).form_valid(form)

    def form_invalid(self, form):
       if self.request.is_ajax():
           self.template_name = self.ajax_form_invalid_template_name
           return self.render_to_response(self.get_context_data(form=form), status=400)
       else:
           return super(PostView, self).form_invalid(form)


@class_view_decorator(login_required)
class Search(BaseListView, FormView):
    form_class = SearchForm
    model = Post
    template_name = 'discussion/search.html'

    def form_invalid(self, form):
        return self.render_to_response(self.get_context_data(form=form,
                                                             object_list=self.get_queryset()))

    def form_valid(self, form):
        """
        Using the name field try to find any posts that match.
        """
        search_term = form.cleaned_data['search']
        object_list = self.model.objects.filter(Q(body__icontains=search_term) |
                                                Q(comment__body__icontains=search_term))
        if form.cleaned_data.get('discussion', None) is not None:
            object_list = object_list.filter(discussion=form.cleaned_data['discussion'])

        return self.render_to_response(self.get_context_data(form=form,
                                                             object_list=object_list.distinct(),
                                                             search_term=search_term))

    def get_context_data(self, **kwargs):
        if 'form' in kwargs:
            kwargs.update({'search_form': kwargs.pop('form')})

        return super(Search, self).get_context_data(**kwargs)

    def get_queryset(self):
        """
        Don't like this currently. I don't want to display all() on a GET but
        don't want to stop subclassers from using this method themselves.
        """
        return self.model.objects.none()

    def get_success_url(self):
        """Defined in case we ever get sent to it by accident"""
        return reverse('discussion_search')



import os 
from django.contrib.auth.models import User
from django.db import models
from django.template.defaultfilters import date, time


class Discussion(models.Model):
    user = models.ForeignKey(User)
    name = models.CharField(max_length=255)
    slug = models.SlugField()
    image = models.ImageField(upload_to='images/discussions', max_length=255, blank=True, null=True)
    description = models.TextField(default='', blank=True, null=True)

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('discussion', [self.slug])

class Post(models.Model):
    discussion = models.ForeignKey(Discussion)
    user = models.ForeignKey(User)
    body = models.TextField()
    attachment = models.FileField(upload_to='uploads/posts', blank=True, null=True)
    time = models.DateTimeField(auto_now_add=True)

    def __unicode__(self):
        return 'Post by {user} at {time} on {date}'.format(
            user=self.user,
            time=time(self.time),
            date=date(self.time),
        )

    @models.permalink
    def get_absolute_url(self):
        return ('discussion_post', [self.discussion.slug, str(self.id)])

    @property
    def attachment_filename(self):
        return self.attachment and os.path.basename(self.attachment.name)

    @property
    def prefix(self):
       return 'post-%d' % (self.pk or 0,)


class Comment(models.Model):
    post = models.ForeignKey(Post)
    user = models.ForeignKey(User)
    body = models.TextField()
    attachment = models.FileField(upload_to='uploads/comments',
                                    blank=True, null=True)
    time = models.DateTimeField(auto_now_add=True)

    @property
    def attachment_filename(self):
        return self.attachment and os.path.basename(self.attachment.name)

    class Meta:
        ordering = ('time',) 

    def __unicode__(self):
        return 'Comment by {user} at {time} on {date}'.format(
            user=self.user,
            time=time(self.time),
            date=date(self.time),
        )



from django.conf.urls.defaults import *
from django.views.generic import RedirectView

from discussion.views import (DiscussionList, DiscussionView,
                              CreatePost, PostView, Search)


urlpatterns = patterns('discussion.views',
    url(r'^$', DiscussionList.as_view(), name='discussion_index'),
    url(r'^search/$', Search.as_view(), name='discussion_search'),
    url(r'^discussions/', include(patterns('',
        url(r'^$', RedirectView.as_view(url='../')),
        url(r'^(?P<slug>[\w-]+)/$', DiscussionView.as_view(), name='discussion'),
        url(r'^(?P<discussion_slug>[\w-]+)/posts/$', RedirectView.as_view(url='../')),
        url(r'^(?P<discussion_slug>[\w-]+)/posts/add/$', CreatePost.as_view(), name='discussion_add_post'),
        url(r'^(?P<discussion_slug>[\w-]+)/posts/(?P<pk>[\d]+)/$', PostView.as_view(), name='discussion_post'),
    )))
)



from django.contrib import admin
from discussion.models import Comment, Discussion, Post


class CommentInline(admin.TabularInline):
    exclude = ('user',)
    extra = 1
    model = Comment


class PostAdmin(admin.ModelAdmin):
    inlines = (CommentInline,)
    list_filter = ('discussion',)


class DiscussionAdmin(admin.ModelAdmin):
    prepopulated_fields = {
        'slug': ('name',)
    }


admin.site.register(Discussion, DiscussionAdmin)
admin.site.register(Post, PostAdmin)
admin.site.register(Comment)
